@@ -15,6 +15,7 @@ from clu.config.ns import unpack_ns, pack_ns, get_ns, compare_ns
 from clu.config.keymapview import KeyMapKeysView, KeyMapItemsView, KeyMapValuesView
 from clu.config.keymapview import NamespaceWalkerKeysView, NamespaceWalkerItemsView
 from clu.config.keymapview import NamespaceWalkerValuesView
+from clu.config.utils import freeze_class
 
 from clu.naming import qualified_import, qualified_name, nameof
 from clu.predicates import (typeof,
@@ -61,6 +62,11 @@ class FrozenKeyMapBase(collections.abc.Mapping,
         """ Iterate over all of the namespaces defined in the mapping. """
         ...
     
+    @abstract
+    def submap(self, *namespaces, unprefixed=False):
+        """ Return a flattened dict containing only the namespaced items. """
+        ...
+    
     @abstract
     def __iter__(self):
         ...
@@ -143,13 +149,13 @@ class FrozenKeyMap(FrozenKeyMapBase):
         """ Used by `clu.config.codecs` to deserialize keymaps """
         return cls(instance_dict)
     
-    def get(self, key, *namespaces, default=NoDefault):
+    def get(self, key, *fragments, default=NoDefault):
         """ Retrieve a (possibly namespaced) value for a given key.
             
             An optional default value may be specified, to be returned
             if the key in question is not found in the mapping.
         """
-        nskey = pack_ns(key, *namespaces)
+        nskey = pack_ns(key, *fragments)
         if default is NoDefault:
             return self[nskey]
         if nskey in self:
@@ -158,12 +164,15 @@ class FrozenKeyMap(FrozenKeyMapBase):
     
     def submap(self, *namespaces, unprefixed=False):
         """ Return a flattened dict containing only the namespaced items. """
+        cls = freeze_class(type(self))
         if unprefixed:
-            return { nskey : self[nskey] for nskey in self if NAMESPACE_SEP not in nskey }
+            return cls({ nskey : self[nskey] for nskey in self if NAMESPACE_SEP not in nskey })
         if not namespaces:
-            return dict(self)
-        prefix = prefix_for(*namespaces)
-        return { nskey : self[nskey] for nskey in self if nskey.startswith(prefix) }
+            return cls(self)
+        out = dict()
+        for namespace in namespaces:
+            out.update({ nskey : self[nskey] for nskey in self if nskey.startswith(namespace) })
+        return cls(out)
     
     def keys(self, *namespaces, unprefixed=False):
         """ Return a namespaced view over either all keys in the mapping,
@@ -198,15 +207,6 @@ class FrozenKeyMap(FrozenKeyMapBase):
     def _get_namespace_foset(self):
         return FlatOrderedSet(get_ns(nskey) for nskey in sorted(self) if NAMESPACE_SEP in nskey)
     
-    def _namespaces(self): # pragma: no cover
-        """ Iterate over all of the namespaces defined in the mapping.
-            
-            This is the generic implementation. It depends on “__iter__(…)”,
-            as implemented by the concrete descendant.
-        """
-        nss = { get_ns(nskey) for nskey in self if NAMESPACE_SEP in nskey }
-        yield from sorted(nss)
-    
     def namespaces(self):
         yield from self._get_namespace_foset()
     
@@ -237,17 +237,17 @@ class KeyMap(KeyMapBase, FrozenKeyMap):
         supra. for further deets, my doggie
     """
     
-    def set(self, key, value, *namespaces):
+    def set(self, key, value, *fragments):
         """ Set a (possibly namespaced) value for a given key. """
-        nskey = pack_ns(key, *namespaces)
+        nskey = pack_ns(key, *fragments)
         self[nskey] = value
     
-    def delete(self, key, *namespaces):
+    def delete(self, key, *fragments):
         """ Delete a (possibly namespaced) value from the mapping. """
-        nskey = pack_ns(key, *namespaces)
+        nskey = pack_ns(key, *fragments)
         del self[nskey]
     
-    def pop(self, key, *namespaces, default=NoDefault):
+    def pop(self, key, *fragments, default=NoDefault):
         """ Pop a (possibly namespaced) value off of the mapping
             and return either that value, or a default value
             if it doesn’t exist – raising a KeyError if no
@@ -256,10 +256,12 @@ class KeyMap(KeyMapBase, FrozenKeyMap):
         # NoDefault never escapes to userland – the call to
         # get(…) will raise the KeyError if NoDefault was passed
         # (Q.v. “get(¬)” definition supra.)
-        value = self.get(key, *namespaces, default=default)
+        value = self.get(key, *fragments, default=default)
+        if value is NoDefault:
+            return None
         if value == default or value is default:
             return value
-        self.delete(key, *namespaces)
+        self.delete(key, *fragments)
         return value
     
     def popitem(self):
@@ -297,7 +299,8 @@ class KeyMap(KeyMapBase, FrozenKeyMap):
             return None
         if not namespaces:
             return super().clear()
-        for nskey in self.submap(*namespaces).keys():
+        submaps = self.submap(*namespaces)
+        for nskey in submaps.keys():
             del self[nskey]
         return None
     
@@ -346,10 +349,10 @@ class NamespaceWalker(FrozenKeyMap):
             Concrete subclasses must implement “walk(…)” such that it iterates
             over all items in a given instance, yielding them in a form like:
             
-                for *namespaces, key, value in self.walk():
+                for *fragments, key, value in self.walk():
                     # …
             
-            … So an item with no namespaces would yield “['key', 'value']”,
+            … So an item with no namespace prefix would yield “['key', 'value']”,
             but one with three would yield “['do', 're', 'me', 'key', 'value']”.
             
             See the “mapwalk(…)” and “envwalk(…)” function implementations,
@@ -371,18 +374,10 @@ class NamespaceWalker(FrozenKeyMap):
         if cls is None:
             from clu.config.keymap import FrozenFlat
             cls = FrozenFlat
-        return cls({ pack_ns(key, *namespaces) : value for *namespaces, key, value in self.walk() })
-    
-    def _namespaces(self): # pragma: no cover
-        """ Iterate over all of the namespaces defined in the mapping. """
-        nss = set()
-        for *namespaces, key, value in self.walk():
-            if namespaces:
-                nss.add(concatenate_ns(*namespaces))
-        yield from sorted(nss)
+        return cls({ pack_ns(key, *fragments) : value for *fragments, key, value in self.walk() })
     
     def _get_namespace_foset(self):
-        return FlatOrderedSet(concatenate_ns(*ns) for *ns, _, _ in self.walk() if ns)
+        return FlatOrderedSet(concatenate_ns(*fragments) for *fragments, _, _ in self.walk() if fragments)
     
     def keys(self, *namespaces, unprefixed=False):
         """ Return a namespaced view over either all keys in the mapping,
@@ -412,25 +407,25 @@ class NamespaceWalker(FrozenKeyMap):
         return NamespaceWalkerValuesView(self, *namespaces)
     
     def __iter__(self):
-        for *namespaces, key, value in self.walk():
-            yield pack_ns(key, *namespaces)
+        for *fragments, key, value in self.walk():
+            yield pack_ns(key, *fragments)
     
     def __len__(self):
         return iterlen(self.walk())
     
     def __contains__(self, nskey):
-        key, namespaces = unpack_ns(nskey)
+        key, fragments = unpack_ns(nskey)
         for *ns, k, value in self.walk():
             if k == key:
-                if compare_ns(ns, namespaces):
+                if compare_ns(ns, fragments):
                     return True
         return False
     
     def __getitem__(self, nskey):
-        key, namespaces = unpack_ns(nskey)
+        key, fragments = unpack_ns(nskey)
         for *ns, k, value in self.walk():
             if k == key:
-                if compare_ns(ns, namespaces):
+                if compare_ns(ns, fragments):
                     return value
         raise KeyError(nskey)
 
