@@ -322,9 +322,9 @@ class Partial(partial):
         """ Initialize a new Partial object, with a predicate and a function """
         # N.B. The real action seems to happen in partial.__new__(…)
         try:
-            super(Partial, self).__init__(*args, **kwargs)
+            super().__init__(*args, **kwargs)
         except:
-            super(Partial, self).__init__() # type: ignore
+            super().__init__() # type: ignore
         
         # Name the Partial instance, as if it’s a phi-type:
         self.__name__ = self.__qualname__ = φ
