@@ -76,6 +76,7 @@ GLOBALS = globals()
 starmods = ('clu.repl.ansi',
             'clu.repl.columnize',
             'clu.repl.modules',
+            'clu.constants.data',
             'clu.config.abc',
             'clu.config.env',
             'clu.config.keymap',
@@ -104,7 +105,6 @@ mods = ('clu',
         'clu.mathematics',
         'clu.dispatch', 'clu.sanitizer', 'clu.fs.pypath',
         'clu.extending', 'clu.shelving.redat',
-        'clu.repr',
         'clu.scripts.ansicolors',
         'clu.scripts.treeline',
         'clu.stdio',
